import codecs
import shutil
import os
import tempfile
from copy import deepcopy

import obeliks

import classla
from classla import Document
from classla.models.common.conll import CoNLLFile

from structure_assignment.constants import *
from structure_assignment.tweak_conllu import tweak as tweak_conllu
from nova_slovnica.translate_jos import translate as translate_jos
from nova_slovnica.conllu_to_xml import convert_file as conllu_to_tei

def create_nlp(resource_directory):
    NLP_CONFIG_MAP['dir'] = resource_directory + '/classla'
    return classla.Pipeline('sl', **NLP_CONFIG_MAP)

class Pipeline:

    def __init__(self, nlp):
        self.nlp = nlp
        self.tmp_directory = tempfile.mkdtemp()
        self.file_map = {key: self.tmp_directory + '/' + FILE_MAP[key] for key in FILE_MAP.keys()}

    def import_file(self, file_name, file_key):
        shutil.copyfile(file_name, self.file_map[file_key])

    def do_tokenise(self):
        input_file_name = self.file_map['strings-list']
        output_file_name = self.file_map['obeliks-tokenised']
        obeliks.run(in_file=input_file_name, out_file=output_file_name, conllu=True)

    def do_tweak_conllu(self):
        input_file_name = self.file_map['obeliks-tokenised']
        output_file_name = self.file_map['obeliks-tweaked']
        tweak_conllu(input_file_name, output_file_name)

    def do_parse(self):
        input_file_name = self.file_map['obeliks-tweaked']
        output_file_name = self.file_map['classla-parsed']
        doc = Document(text=None)
        conll_file = CoNLLFile(filename=input_file_name)
        doc.conll_file = conll_file
        result = nlp(doc)
        result.conll_file.write_conll(output_file_name)

    def do_translate_jos(self):
        input_file_name = self.file_map['classla-parsed']
        dictionary_file_name = resource_directory + '/dict.xml'
        output_file_name = self.file_map['classla-translated']
        translate_jos(input_file_name, dictionary_file_name, output_file_name)

    def do_conllu_to_tei(self):
        input_file_name = self.file_map['classla-translated']
        output_file_name = self.file_map['tei-initial']
        conllu_to_tei(input_file_name, output_file_name)

    def export_file(self, file_name, file_key):
        shutil.copyfile(self.file_map[file_key], file_name)

    def cleanup(self):
        shutil.rmtree(self.tmp_directory, True)
